@@ -1,4 +1,4 @@
-/* $Id: xfburn-main-window.c 22665 2006-08-05 12:44:03Z pollux $ */
+/* $Id: xfburn-main-window.c 5095 2008-07-15 15:05:01Z squisher $ */
 /*
  * Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -28,14 +28,18 @@
 #include <exo/exo.h>
 
 #include "xfburn-main-window.h"
+#include "xfburn-device-list.h"
 #include "xfburn-preferences-dialog.h"
 #include "xfburn-file-browser.h"
 #include "xfburn-compositions-notebook.h"
-#include "xfburn-blank-cd-dialog.h"
+#include "xfburn-blank-dialog.h"
+#include "xfburn-format-dvd-dialog.h"
 #include "xfburn-copy-cd-dialog.h"
+#include "xfburn-copy-dvd-dialog.h"
 #include "xfburn-burn-image-dialog.h"
 #include "xfburn-progress-dialog.h"
 #include "xfburn-settings.h"
+#include "xfburn-main.h"
 
 #define XFBURN_MAIN_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), XFBURN_TYPE_MAIN_WINDOW, XfburnMainWindowPrivate))
 
@@ -51,6 +55,10 @@ typedef struct {
   
   GtkWidget *file_browser;
   GtkWidget *compositions_notebook;
+
+  gboolean support_cdr;
+  gboolean support_cdrw;
+  XfburnWelcomeTab *welcome_tab;
 } XfburnMainWindowPrivate;
 
 /* prototypes */
@@ -65,18 +73,24 @@ static void action_about (GtkAction *, XfburnMainWindow *);
 static void action_preferences (GtkAction *, XfburnMainWindow *);
 
 static void action_new_data_composition (GtkAction *, XfburnMainWindow *);
-static void action_new_audio_composition (GtkAction *, XfburnMainWindow *);
+//static void action_new_audio_composition (GtkAction *, XfburnMainWindow *);
 
+/*
 static void action_load (GtkAction *, XfburnMainWindow *);
 static void action_save (GtkAction *, XfburnMainWindow *);
 static void action_save_as (GtkAction *, XfburnMainWindow *);
+*/
 static void action_close (GtkAction *, XfburnMainWindow *);
 static void action_quit (GtkAction *, XfburnMainWindow *);
 
-static void action_blank_cd (GtkAction *, XfburnMainWindow *);
+static void action_blank (GtkAction *, XfburnMainWindow *);
 static void action_copy_cd (GtkAction *, XfburnMainWindow *);
 static void action_burn_image (GtkAction *, XfburnMainWindow *);
 
+static void action_format_dvd (GtkAction *, XfburnMainWindow *);
+static void action_copy_dvd (GtkAction *, XfburnMainWindow *);
+static void action_burn_dvd_image (GtkAction *, XfburnMainWindow *);
+
 static void action_refresh_directorybrowser (GtkAction *, XfburnMainWindow *);
 static void action_show_filebrowser (GtkToggleAction *, XfburnMainWindow *);
 static void action_show_toolbar (GtkToggleAction * action, XfburnMainWindow * window);
@@ -111,14 +125,19 @@ static const GtkActionEntry action_entries[] = {
   {"help-menu", NULL, N_("_Help"), NULL,},
   {"about", GTK_STOCK_ABOUT, N_("_About"), NULL, N_("Display information about Xfburn"),
    G_CALLBACK (action_about),},
-  {"blank-cd", "xfburn-blank-cdrw", N_("Blank CD-RW"), NULL, N_("Blank CD-RW"),
-   G_CALLBACK (action_blank_cd),},
-  {"format-dvd", "xfburn-blank-dvdrw", N_("Format DVD-RW"), NULL, N_("Format DVD-RW"),},
+  {"blank-disc", "xfburn-blank-cdrw", N_("Blank CD-RW"), NULL, N_("Blank CD-RW"),
+   G_CALLBACK (action_blank),},
   {"copy-data", "xfburn-data-copy", N_("Copy Data CD"), NULL, N_("Copy Data CD"),
    G_CALLBACK (action_copy_cd),},
   {"copy-audio", "xfburn-audio-copy", N_("Copy Audio CD"), NULL, N_("Copy Audio CD"),},
-  {"burn-cd", "xfburn-burn-cd", N_("Burn CD Image"), NULL, N_("Burn CD Image"),
+  {"burn-image", "xfburn-burn-cd", N_("Burn Image"), NULL, N_("Burn Image"),
    G_CALLBACK (action_burn_image),},
+  {"format-dvd", "xfburn-format-dvdrw", N_("Format DVD+RW"), NULL, N_("Format DVD+RW"),
+   G_CALLBACK (action_format_dvd),},
+  {"copy-dvd", "xfburn-data-copy", N_("Copy DVD"), NULL, N_("Copy DVD"),
+   G_CALLBACK (action_copy_dvd),},
+  {"burn-dvd", "xfburn-burn-image", N_("Burn DVD Image"), NULL, N_("Burn DVD Image"),
+   G_CALLBACK (action_burn_dvd_image),},
 };
 
 static const GtkToggleActionEntry toggle_action_entries[] = {
@@ -133,11 +152,13 @@ static const gchar *toolbar_actions[] = {
 /*  "load-composition",
   "save-composition",
   "close-composition",*/
-  "blank-cd",
-  //"format-dvd",
+  "blank-disc",
   "copy-data",
   //"copy-audio",
-  "burn-cd",
+  "burn-image",
+  "format-dvd",
+  "copy-dvd",
+  "burn-dvd",
   "refresh",
   "about",
   "preferences",
@@ -295,6 +316,7 @@ xfburn_main_window_init (XfburnMainWindow * mainwin)
   priv->compositions_notebook = xfburn_compositions_notebook_new ();
   gtk_widget_show (priv->compositions_notebook);
   gtk_paned_add2 (GTK_PANED (priv->vpaned), priv->compositions_notebook);
+  priv->welcome_tab = xfburn_compositions_notebook_add_welcome_tab (XFBURN_COMPOSITIONS_NOTEBOOK (priv->compositions_notebook), mainwin);
   
   gtk_paned_set_position (GTK_PANED (priv->vpaned), xfburn_settings_get_int ("vpaned-position", 200));
 
@@ -353,11 +375,11 @@ cb_delete_main_window (XfburnMainWindow * mainwin, GdkEvent * event, XfburnMainW
 
 /* actions */
 static void
-action_blank_cd (GtkAction * action, XfburnMainWindow * window)
+action_blank (GtkAction * action, XfburnMainWindow * window)
 {
   GtkWidget *dialog;
   
-  dialog = xfburn_blank_cd_dialog_new ();
+  dialog = xfburn_blank_dialog_new ();
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
   gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
@@ -365,29 +387,58 @@ action_blank_cd (GtkAction * action, XfburnMainWindow * window)
 
 static void action_copy_cd (GtkAction *action, XfburnMainWindow *window)
 {
+  /*
   GtkWidget *dialog;
-  
   dialog = xfburn_copy_cd_dialog_new ();
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
   gtk_dialog_run (GTK_DIALOG (dialog)); 
   gtk_widget_destroy (dialog);
+  */
 }
 
 static void
 action_burn_image (GtkAction * action, XfburnMainWindow * window)
 {
   GtkWidget *dialog;
-  
+
   dialog = xfburn_burn_image_dialog_new ();
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
   gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
 }
 
+static void
+action_format_dvd (GtkAction * action, XfburnMainWindow * window)
+{
+  GtkWidget *dialog;
+  
+  dialog = xfburn_format_dvd_dialog_new ();
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+}
+
+static void
+action_copy_dvd (GtkAction * action, XfburnMainWindow * window)
+{
+  GtkWidget *dialog;
+  
+  dialog = xfburn_copy_dvd_dialog_new ();
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
+  gtk_dialog_run (GTK_DIALOG (dialog)); 
+  gtk_widget_destroy (dialog);
+}
+
+static void
+action_burn_dvd_image (GtkAction * action, XfburnMainWindow * window)
+{
+}
+
+/*
 static void
 action_load (GtkAction *action, XfburnMainWindow * window)
 {
-  XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
+  //XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
   
   //xfburn_data_composition_load_from_file (XFBURN_DATA_COMPOSITION (priv->data_composition), "/tmp/test.xml");
 }
@@ -403,9 +454,10 @@ action_save (GtkAction *action, XfburnMainWindow * window)
 static void
 action_save_as (GtkAction *action, XfburnMainWindow * window)
 {
-  XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
+  //XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
   
 }
+*/
 
 static void
 action_close (GtkAction *action, XfburnMainWindow * window)
@@ -418,23 +470,23 @@ action_close (GtkAction *action, XfburnMainWindow * window)
 static void
 action_new_data_composition (GtkAction *action, XfburnMainWindow * window)
 {
-  XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
- 
-  xfburn_compositions_notebook_add_composition (XFBURN_COMPOSITIONS_NOTEBOOK (priv->compositions_notebook), XFBURN_DATA_COMPOSITION);
+  xfburn_main_window_add_data_composition_with_files (window, 0, NULL);
 }
 
+/*
 static void
 action_new_audio_composition (GtkAction *action, XfburnMainWindow * window)
 {
   XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
  
 }
+*/
 
 static void
 action_quit (GtkAction * action, XfburnMainWindow * window)
 {
-  if (xfce_confirm (_("Are sure you want to quit?"), GTK_STOCK_QUIT, _("_Quit")))
-    gtk_main_quit ();
+  // if (xfce_confirm (_("Are sure you want to quit?"), GTK_STOCK_QUIT, _("_Quit")))
+  gtk_main_quit ();
 }
 
 static void
@@ -442,6 +494,7 @@ action_about (GtkAction * action, XfburnMainWindow * window)
 {
   XfceAboutInfo *info;
   GtkWidget *dialog;
+  gint x, y;
   GdkPixbuf *icon;
   guint n;
 
@@ -449,32 +502,53 @@ action_about (GtkAction * action, XfburnMainWindow * window)
   {
     gchar *name, *email, *language;
   } translators[] = {
+    {"Mohamed Magdy", "mohamed.m.k@gmail.com", "ar",},
     {"Pau Rul lan Ferragut", "paurullan@bulma.net", "ca",},
     {"Michal Várady", "miko.vaji@gmail.com", "cs",},
     {"Enrico Tröger", "enrico.troeger@uvena.de", "de",},
+    {"Fabian Nowak", "timstery@arcor.de", "de",},
+    {"Nico Schümann", "nico@prog.nico22.de", "de",},
     {"Stavros Giannouris", "stavrosg2002@freemail.gr", "el",},
+    {"Jeff Bailes", "thepizzaking@gmail.com", "en_GB",},
+    {"Diego Rodriguez", "dieymir@yahoo.es", "es",},
     {"Piarres Beobide", "pi@beobide.net", "eu",},
-    {"Jari Rahkonen", "jari.rahkonen@pp2.inet.fi", "fi",},
+    {"Jari Rahkonen", "jari.rahkonen@pp1.inet.fi", "fi",},
     {"Etienne Collet", "xanaxlnx@gmail.com", "fr",},
-    {"Maximilian Schleiss", "maxschleiss@bluewin.ch", "fr",},
-    {"Attila SZERVÑC", "sas@321.hu", "hu",},
+    {"Maximilian Schleiss", "maximilian@xfce.org", "fr",},
+    {"Attila Szervác", "sas@321.hu", "hu",},
     {"Daichi Kawahata", "daichi@xfce.org", "ja",},
+    {"ByungHyun Choi", "byunghyun.choi@debianusers.org", "kr",},
     {"Mantas", "mantaz@users.sourceforge.net", "lt",},
+    {"Rihards Prieditis", "RPrieditis@inbox.lv", "lv",},
+    {"Terje Uriansrud", "ter@operamail.com", "nb_NO",},
+    {"Stephan Arts", "psybsd@gmail.com", "nl",},
     {"Szymon Kałasz", "szymon_maestro@gazeta.pl", "pl",},
+    {"Fábio Nogueira", "deb-user-ba@ubuntu.com", "pt_BR",},
+    {"Og Maciel", "omaciel@xfce.org", "pt_BR",},
+    {"Nuno Miguel", "nunis@netcabo.pt", "pt_PT",},
     {"Sergey Fedoseev", "fedoseev.sergey@gmail.com", "ru",},
+    {"Besnik Bleta", "besnik@programeshqip.org", "sq",},
     {"Maxim V. Dziumanenko", "mvd@mylinux.com.ua", "uk",},
+    {"Dmitry Nikitin", "", "uk",},
+    {"ﻢﺤﻣﺩ ﻊﻠﻳ ﺎﻠﻤﻜﻳ", "makki.ma@gmail.com", "ur",},
+    {"正龙 赵", "zzl6612@yahoo.com.cn", "zh_CN",},
     {"Cosmo Chene", "cosmolax@gmail.com", "zh_TW",},
   };
 
-  icon = xfce_themed_icon_load (GTK_STOCK_CDROM, 48);
-  //if (G_UNLIKELY (icon == NULL))
-  //icon = gdk_pixbuf_new_from_file (DATADIR "/icons/hicolor/48x48/apps/Terminal.png", NULL);
-  
-  info = xfce_about_info_new ("Xfburn", VERSION, _("Another cd burning tool"),
-                              XFCE_COPYRIGHT_TEXT ("2005-2006", "Jean-François Wauthy"), XFCE_LICENSE_GPL);
-  xfce_about_info_set_homepage (info, "http://www.xfce.org/");
+  gtk_icon_size_lookup (GTK_ICON_SIZE_DIALOG, &x, &y);
+  icon = xfce_themed_icon_load ("media-optical", x);
+  if (!icon)
+    icon = xfce_themed_icon_load ("media-cdrom", x);
+  if (!icon)
+    icon = xfce_themed_icon_load (GTK_STOCK_CDROM, x);
+
+  info = xfce_about_info_new ("Xfburn", VERSION, _("Another cd burning GUI"),
+                              XFCE_COPYRIGHT_TEXT ("2005-2008", "Jean-François Wauthy, Mario Đanić, David Mohr"), XFCE_LICENSE_GPL);
+  xfce_about_info_set_homepage (info, "http://www.xfce.org/projects/xfburn");
   xfce_about_info_add_credit (info, "Jean-François Wauthy", "pollux@xfce.org", _("Author/Maintainer"));
-  //xfce_about_info_add_credit (info, "Francois Le Clainche", "fleclainche@wanadoo.fr", _("Icon Designer"));
+  xfce_about_info_add_credit (info, "Mario Đanić", "mario@libburnia-project.org", _("Author/Maintainer"));
+  xfce_about_info_add_credit (info, "David Mohr", "david@mcbf.net", _("Author/Maintainer"));
+  
 
   for (n = 0; n < G_N_ELEMENTS (translators); ++n) {
     gchar *s;
@@ -551,8 +625,6 @@ GtkWidget *
 xfburn_main_window_new (void)
 {
   GtkWidget *obj;
-  XfburnMainWindow *win;
-  GtkAction *action;
 
   if (G_UNLIKELY (instance)) {
     g_error ("An instance of XfburnMainWindow has already been created");
@@ -562,7 +634,10 @@ xfburn_main_window_new (void)
   obj = g_object_new (xfburn_main_window_get_type (), NULL);
   
   if (obj) {
+    XfburnMainWindow *win;
     XfburnMainWindowPrivate *priv;
+    GtkAction *action;
+    GList *device = NULL;
     
     instance = win = XFBURN_MAIN_WINDOW (obj);
     priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (win);
@@ -574,8 +649,37 @@ xfburn_main_window_new (void)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), xfburn_settings_get_boolean ("show-toolbar", TRUE));
    /* action = gtk_action_group_get_action (priv->action_group, "save-composition");
     gtk_action_set_sensitive (GTK_ACTION (action), FALSE);*/
+
+    /* disable action that cannot be used due to device */
+    device = xfburn_device_list_get_list ();
+
+    while (device != NULL) {
+      XfburnDevice *device_info = (XfburnDevice *) device->data;
+
+      if (device_info->cdr)
+	priv->support_cdr = TRUE;
+      if (device_info->cdrw)
+	priv->support_cdrw = TRUE;
+
+      device = g_list_next (device);
+    }
+
+    if (!priv->support_cdr) {
+      action = gtk_action_group_get_action (priv->action_group, "copy-data");
+      gtk_action_set_sensitive (action, FALSE);
+      action = gtk_action_group_get_action (priv->action_group, "copy-audio");
+      gtk_action_set_sensitive (action, FALSE);
+      action = gtk_action_group_get_action (priv->action_group, "burn-image");
+      gtk_action_set_sensitive (action, FALSE);
+    }
+    if (!priv->support_cdrw) {
+      action = gtk_action_group_get_action (priv->action_group, "blank-disc");
+      gtk_action_set_sensitive (action, FALSE);
+    }
+
+    g_object_set (priv->welcome_tab, "action_group", priv->action_group, NULL);
   }
-  
+
   return obj;
 }
 
@@ -603,3 +707,29 @@ xfburn_main_window_get_file_browser (XfburnMainWindow *window)
   
   return XFBURN_FILE_BROWSER (priv->file_browser);
 }
+
+void 
+xfburn_main_window_add_data_composition_with_files (XfburnMainWindow *window, int filec, char **filenames)
+{
+  XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
+  XfburnComposition *comp;
+ 
+  comp = xfburn_compositions_notebook_add_composition (XFBURN_COMPOSITIONS_NOTEBOOK (priv->compositions_notebook), XFBURN_DATA_COMPOSITION);
+  xfburn_data_composition_add_files (XFBURN_DATA_COMPOSITION (comp), filec, filenames);
+}
+
+gboolean
+xfburn_main_window_support_cdr (XfburnMainWindow *window)
+{
+  XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
+
+  return priv->support_cdr;
+}
+
+gboolean
+xfburn_main_window_support_cdrw (XfburnMainWindow *window)
+{
+  XfburnMainWindowPrivate *priv = XFBURN_MAIN_WINDOW_GET_PRIVATE (window);
+
+  return priv->support_cdrw;
+}
