@@ -1,4 +1,4 @@
-/* $Id: xfburn-preferences-dialog.c 22714 2006-08-11 16:02:53Z pollux $ */
+/* $Id: xfburn-preferences-dialog.c 5095 2008-07-15 15:05:01Z squisher $ */
 /*
  *  Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -25,7 +25,7 @@
 #include <exo/exo.h>
 
 #include "xfburn-preferences-dialog.h"
-#include "xfburn-global.h"
+#include "xfburn-device-list.h"
 #include "xfburn-utils.h"
 #include "xfburn-settings.h"
 
@@ -43,6 +43,8 @@ typedef struct
 
   GtkWidget *treeview_devices;
   GtkWidget *button_scan;
+  GtkWidget *check_empty_speed_list;
+  GtkWidget *scale_fifo;
 } XfburnPreferencesDialogPrivate;
 
 /* prototypes */
@@ -60,6 +62,18 @@ enum {
   SETTINGS_LIST_N_COLUMNS,
 };
 
+enum
+{
+  DEVICE_LIST_COLUMN_ICON,
+  DEVICE_LIST_COLUMN_NAME,
+  DEVICE_LIST_COLUMN_NODE,
+  DEVICE_LIST_COLUMN_CDR,
+  DEVICE_LIST_COLUMN_CDRW,
+  DEVICE_LIST_COLUMN_DVDR,
+  DEVICE_LIST_COLUMN_DVDRAM,
+  DEVICE_LIST_N_COLUMNS
+};
+
 typedef struct
 {
   XfburnPreferencesDialog *object;
@@ -108,11 +122,12 @@ xfburn_preferences_dialog_init (XfburnPreferencesDialog * obj)
   GtkBox *box = GTK_BOX (GTK_DIALOG (obj)->vbox);
   XfburnPreferencesDialogPrivate *priv = XFBURN_PREFERENCES_DIALOG_GET_PRIVATE (obj);
   
-  GtkWidget *vbox, *vbox2, *hbox;
+  GtkWidget *vbox, *vbox2, *vbox3, *hbox;
   GtkWidget *label;
   GtkWidget *frame;
   GtkWidget *scrolled_window;
   GtkListStore *icon_store, *store;
+  gint x,y;
   GdkPixbuf *icon = NULL;
   GtkTreeIter iter;
   GtkTreeViewColumn *column_name;
@@ -231,7 +246,7 @@ xfburn_preferences_dialog_init (XfburnPreferencesDialog * obj)
   gtk_box_pack_start (GTK_BOX (vbox2), scrolled_window, TRUE, TRUE, BORDER);
 
   store = gtk_list_store_new (DEVICE_LIST_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING,
-                              G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
+                              G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
   priv->treeview_devices = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), DEVICE_LIST_COLUMN_NAME, GTK_SORT_ASCENDING);
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (priv->treeview_devices), TRUE);
@@ -254,8 +269,6 @@ xfburn_preferences_dialog_init (XfburnPreferencesDialog * obj)
   gtk_tree_view_column_set_attributes (column_name, cell_name, "text", DEVICE_LIST_COLUMN_NAME, NULL);
 
   gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview_devices), column_name);
-  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->treeview_devices), -1, _("Id"),
-                                               gtk_cell_renderer_text_new (), "text", DEVICE_LIST_COLUMN_ID, NULL);
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->treeview_devices), -1, _("Node"),
                                                gtk_cell_renderer_text_new (), "text", DEVICE_LIST_COLUMN_NODE, NULL);
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->treeview_devices), -1, _("Write CD-R"),
@@ -279,19 +292,45 @@ xfburn_preferences_dialog_init (XfburnPreferencesDialog * obj)
   g_signal_connect (G_OBJECT (priv->button_scan), "clicked", G_CALLBACK (scan_button_clicked_cb), obj);
   gtk_widget_show (priv->button_scan);
 
-  icon = gtk_widget_render_icon (GTK_WIDGET (priv->icon_bar),
-                                 GTK_STOCK_CDROM,
-                                 GTK_ICON_SIZE_DIALOG,
-                                 NULL);
+  gtk_icon_size_lookup (GTK_ICON_SIZE_DIALOG, &x, &y);
+  icon = xfce_themed_icon_load ("media-optical", x);
+  if (!icon)
+    icon = xfce_themed_icon_load ("media-cdrom", x);
+  if (!icon)
+    icon = xfce_themed_icon_load (GTK_STOCK_CDROM, x);
+
+  
   gtk_list_store_append (icon_store, &iter);
   gtk_list_store_set (icon_store, &iter,
                       SETTINGS_LIST_PIXBUF_COLUMN, icon,
                       SETTINGS_LIST_TEXT_COLUMN, _("Devices"),
                       SETTINGS_LIST_INDEX_COLUMN, index,
                       -1);
-  g_object_unref (G_OBJECT (icon));
+  if (icon)
+    g_object_unref (G_OBJECT (icon));
   
   exo_mutual_binding_new (G_OBJECT (priv->notebook), "page", G_OBJECT (priv->icon_bar), "active");
+
+
+  /* below the device list */
+  priv->check_empty_speed_list = gtk_check_button_new_with_mnemonic (_("Show _empty speed list bug warning"));
+  gtk_box_pack_start (GTK_BOX (vbox2), priv->check_empty_speed_list, FALSE, FALSE, BORDER);
+  gtk_widget_show (priv->check_empty_speed_list);
+
+  /* fifo */
+  vbox3 = gtk_vbox_new (FALSE, 0);
+  gtk_widget_show (vbox3);
+
+  frame = xfce_create_framebox_with_content (_("FIFO buffer size (in kb)"), vbox3);
+  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, BORDER);
+  gtk_widget_show (frame);
+
+  priv->scale_fifo = gtk_hscale_new_with_range (0.0, 8192.0, 32.0);
+  gtk_scale_set_value_pos (GTK_SCALE (priv->scale_fifo), GTK_POS_LEFT);
+  gtk_range_set_value (GTK_RANGE (priv->scale_fifo), 0);
+  gtk_box_pack_start (GTK_BOX (vbox3), priv->scale_fifo, FALSE, FALSE, BORDER/2);
+  gtk_widget_show (priv->scale_fifo);
+
   
   /* action buttons */
   button_close = gtk_button_new_from_stock ("gtk-close");
@@ -326,6 +365,10 @@ xfburn_preferences_dialog_load_settings (XfburnPreferencesDialog * dialog)
                                 xfburn_settings_get_boolean ("show-hidden-files", FALSE));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->check_show_human_readable),
                                 xfburn_settings_get_boolean ("human-readable-units", TRUE));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->check_empty_speed_list),
+                                xfburn_settings_get_boolean ("show-empty-speed-list-notice", TRUE));
+  gtk_range_set_value (GTK_RANGE (priv->scale_fifo),
+                                (double) xfburn_settings_get_int ("fifo-size", XFBURN_FIFO_DEFAULT_SIZE));
 }
 
 static void
@@ -344,6 +387,10 @@ xfburn_preferences_dialog_save_settings (XfburnPreferencesDialog *dialog)
                                gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_show_hidden)));
   xfburn_settings_set_boolean ("human-readable-units", 
                                gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_show_human_readable)));
+  xfburn_settings_set_boolean ("show-empty-speed-list-notice", 
+                               gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_empty_speed_list)));
+  xfburn_settings_set_int ("fifo-size", 
+                               ((((int) gtk_range_get_value (GTK_RANGE (priv->scale_fifo)) / 32) * 32))); /* this should round to multiples of 1024 */
 }
 
 static void
@@ -357,7 +404,7 @@ refresh_devices_list (XfburnPreferencesDialog * dialog)
 
   gtk_list_store_clear (GTK_LIST_STORE (model));
 
-  device = list_devices;
+  device = xfburn_device_list_get_list ();
   while (device) {
     GtkTreeIter iter;
     XfburnDevice *device_data;
@@ -367,7 +414,6 @@ refresh_devices_list (XfburnPreferencesDialog * dialog)
     gtk_list_store_append (GTK_LIST_STORE (model), &iter);
     gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                         DEVICE_LIST_COLUMN_NAME, device_data->name,
-                        DEVICE_LIST_COLUMN_ID, device_data->id,
                         DEVICE_LIST_COLUMN_NODE, device_data->node_path,
                         DEVICE_LIST_COLUMN_CDR, device_data->cdr,
                         DEVICE_LIST_COLUMN_CDRW, device_data->cdrw,
@@ -388,7 +434,7 @@ xfburn_preferences_dialog_response_cb (XfburnPreferencesDialog * dialog, guint r
 static void
 scan_button_clicked_cb (GtkWidget * button, gpointer user_data)
 {
-  xfburn_scan_devices ();
+  xfburn_device_list_init ();
   refresh_devices_list (user_data);
 }
 
