@@ -1,6 +1,6 @@
-/* $Id: xfburn-stock.h 21812 2006-05-24 16:28:23Z pollux $ */
+/* $Id: xfburn-stock.h 5095 2008-07-15 15:05:01Z squisher $ */
 /*
- * Copyright (c) 2006 Jean-Franois Wauthy (pollux@xfce.org)
+ * Copyright (c) 2006 Jean-François Wauthy (pollux@xfce.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ G_BEGIN_DECLS;
 
 #define XFBURN_STOCK_AUDIO_COPY     "xfburn-audio-copy"
 #define XFBURN_STOCK_BLANK_CDRW     "xfburn-blank-cdrw"
-#define XFBURN_STOCK_BLANK_DVDRW    "xfburn-blank-dvdrw"
+#define XFBURN_STOCK_FORMAT_DVDRW   "xfburn-format-dvdrw"
 #define XFBURN_STOCK_BURN_CD        "xfburn-burn-cd"
 #define XFBURN_STOCK_CD             "xfburn-cd"
 #define XFBURN_STOCK_DATA_COPY      "xfburn-data-copy"
