@@ -1,4 +1,4 @@
-/* $Id: xfburn-utils.h 21873 2006-05-27 19:17:58Z pollux $ */
+/* $Id: xfburn-utils.h 5095 2008-07-15 15:05:01Z squisher $ */
 /*
  * Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -24,23 +24,17 @@
 #include <config.h>
 #endif /* !HAVE_CONFIG_H */
 
-#include <linux/cdrom.h>
+#include <gtk/gtk.h>
 
 #include "xfburn-global.h"
 
 void xfburn_busy_cursor (GtkWidget *);
 void xfburn_default_cursor (GtkWidget *);
 
-void xfburn_device_content_free (XfburnDevice * device, gpointer user_data);
-void xfburn_device_free (XfburnDevice * device);
-XfburnDevice * xfburn_device_lookup_by_name (const gchar * name);
-gint xfburn_device_query_cdstatus (XfburnDevice * device);
-gchar * xfburn_device_cdstatus_to_string (gint status);
-
-void xfburn_scan_devices ();
-
 gchar *xfburn_humanreadable_filesize (guint64);
 guint64 xfburn_calc_dirsize (const gchar *);
 
 void xfburn_browse_for_file (GtkEntry *entry, GtkWindow *parent);
+
+gboolean xfburn_ask_yes_no (GtkMessageType type, const gchar *primary_text, const gchar *secondary_text);
 #endif
